# generated by datamodel-codegen:
#   timestamp: 2025-10-13T03:48:19+00:00

from __future__ import annotations

from typing import Any

from pydantic import BaseModel


class Docs(BaseModel):  # type: ignore
    isSharedWithUser: bool | None = None
    orgId: int | None = None
    templateId: str | None = None
    name: str | None = None
    token: str | None = None
    deviceId: int | None = None
    productId: int | None = None
    ownerId: int | None = None
    productName: str | None = None
    isLocked: bool | None = None
    email: str | None = None
    userId: int | None = None
    icon: str | None = None
    color: str | None = None
    type: str | None = None


class EviqoDeviceQueryModel(BaseModel):  # type: ignore
    count: int
    docs: list[list[list[Any] | int | str | Docs]]
